package no.ntnu.stianren.applist;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

public class SettingsProvider extends ContentProvider 
{

	public static final String PROVIDER_NAME = "no.ntnu.provider.settings";
	public static final Uri CONTENT_URI = Uri.parse("content://" + PROVIDER_NAME + "/settings");
	
	public static final String _ID = "_id";
	public static final String AP = "ap";
	public static final String FAKE = "fake";
	public static final String FAKE_SETTING = "fake_setting";
	
	private static final int PERMISSIONS = 1;
	private static final int PERMISSION = 2;
	
	private static final UriMatcher uriMatcher;
	
	static
	{
		uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
		uriMatcher.addURI(PROVIDER_NAME, "permissions", PERMISSIONS);
		uriMatcher.addURI(PROVIDER_NAME, "permissions/#", PERMISSION);
	}
	
	/*
	 * Private helper-class for database usage
	 */
	
	private SQLiteDatabase settingsDB;
	private static final String DATABASE_NAME = "Settings";
	private static final String DATABASE_TABLE = "accessPermissions";
	private static final int DATABASE_VERSION = 1;
	private static final String DATABASE_CREATE = 
		"create table " + DATABASE_TABLE + 
		" (_id integer primary key autoincrement, " +
		"AP text not null, fake integer not null, fake_setting integer not null);";
	
	private static class DatabaseHelper extends SQLiteOpenHelper
	{
		DatabaseHelper(Context context)
		{
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}
		
		@Override
		public void onCreate(SQLiteDatabase db)
		{
			db.execSQL(DATABASE_CREATE);
		}
		
		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion)
		{
			Log.w("Content provider database", "Upgrading database from version " + oldVersion + " to " + newVersion + " - old data will be destroyed!");
			db.execSQL("DROP TABLE IF EXISTS accessPermissions");
			onCreate(db);
		}
	}
	
	@Override
	public String getType(Uri uri)
	{
		switch (uriMatcher.match(uri))
		{
		case PERMISSIONS:
			return "vnd.android.cursor.dir/vnd.ntnu.permissions ";
		case PERMISSION:
			return "vnd.android.cursor.item/vnd.ntnu.permissions ";
		default:
			throw new IllegalArgumentException("Unsupported URI: " + uri);
		}
	}
	
	@Override
	public boolean onCreate() 
	{
		Context context = getContext();
		DatabaseHelper dbHelper = new DatabaseHelper(context);
		settingsDB = dbHelper.getWritableDatabase();
		return (settingsDB == null)? false:true;
	}
	
	@Override
	public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) 
	{
		SQLiteQueryBuilder sqlBuilder = new SQLiteQueryBuilder();
	    sqlBuilder.setTables(DATABASE_TABLE);
	       
	    if (uriMatcher.match(uri) == PERMISSION)
	        //---if getting a particular permission---
	    	sqlBuilder.appendWhere(_ID + " = " + uri.getPathSegments().get(1));                
	       
	    if (sortOrder==null || sortOrder=="")
	    	sortOrder = AP;
	   
	    Cursor c = sqlBuilder.query(settingsDB, projection, selection, selectionArgs, null, null, sortOrder);
	   
	    //---register to watch a content URI for changes---
	    c.setNotificationUri(getContext().getContentResolver(), uri);
	    return c;
	}

	@Override
	public Uri insert(Uri uri, ContentValues values) 
	{
	    //---add a new permission---
		long rowID = settingsDB.insert(DATABASE_TABLE, "", values);
	           
	    //---if added successfully---
	    if (rowID>0)
	    {
	    	Uri _uri = ContentUris.withAppendedId(CONTENT_URI, rowID);
	        getContext().getContentResolver().notifyChange(_uri, null);    
	        return _uri;                
	    }        
	    throw new SQLException("Failed to insert row into " + uri);
	}
	
	@Override
	public int delete(Uri arg0, String arg1, String[] arg2) 
	{
		// arg0 = uri 
	    // arg1 = selection
	    // arg2 = selectionArgs
	    int count=0;
	    switch (uriMatcher.match(arg0))
	    {
	    case PERMISSIONS:
	        count = settingsDB.delete(DATABASE_TABLE, arg1, arg2);
	        break;
	    case PERMISSION:
	    	String id = arg0.getPathSegments().get(1);
	    	count = settingsDB.delete(DATABASE_TABLE, _ID + " = " + id + (!TextUtils.isEmpty(arg1) ? " AND (" + arg1 + ')' : ""), arg2);
	    	break;
	    default: 
	    	throw new IllegalArgumentException("Unknown URI " + arg0);    
	    }       
	    getContext().getContentResolver().notifyChange(arg0, null);
	    return count;      
	}

	@Override
	public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) 
	{
		int count = 0;
		switch (uriMatcher.match(uri))
		{
		case PERMISSIONS:
			count = settingsDB.update(DATABASE_TABLE, values, selection, selectionArgs);
			break;
		case PERMISSION:                
			count = settingsDB.update(DATABASE_TABLE, values, _ID + " = " + uri.getPathSegments().get(1) + (!TextUtils.isEmpty(selection) ? " AND (" + selection + ')' : ""), selectionArgs);
			break;
		default: throw new IllegalArgumentException("Unknown URI " + uri);    
		}       
		getContext().getContentResolver().notifyChange(uri, null);
		return count;
	}

}
